/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2018 Research Group Biomedical Physics,
 * Max-Planck-Institute for Dynamics and Self-Organization Göttingen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */
package org.caosdb.server.database.backend.implementation.MySQL;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import org.caosdb.server.database.access.Access;
import org.caosdb.server.database.backend.interfaces.RetrieveAllImpl;
import org.caosdb.server.database.exceptions.TransactionException;
import org.caosdb.server.entity.Role;

public class MySQLRetrieveAll extends MySQLTransaction implements RetrieveAllImpl {

  public MySQLRetrieveAll(final Access access) {
    super(access);
  }

  public static final String STMT_GET_ALL_HEAD = "Select id from entities where id > 99";
  public static final String STMT_ENTITY_WHERE_CLAUSE =
      " AND ( role=? OR role='"
          + Role.RecordType
          + "' OR role='"
          + Role.Property
          + "' OR role='"
          + Role.File
          + "'"
          + " )";
  public static final String STMT_OTHER_ROLES = " AND role=?";

  @Override
  public List<Integer> execute(final String role) throws TransactionException {
    try {
      final String STMT_GET_ALL =
          STMT_GET_ALL_HEAD
              + (role.equalsIgnoreCase("ENTITY") ? STMT_ENTITY_WHERE_CLAUSE : STMT_OTHER_ROLES);
      final PreparedStatement stmt = prepareStatement(STMT_GET_ALL);

      if (role.equalsIgnoreCase("ENTITY")) {
        stmt.setString(1, Role.Record.toString());

      } else {
        stmt.setString(1, role);
      }

      final ResultSet rs = stmt.executeQuery();
      try {
        final ArrayList<Integer> ret = new ArrayList<Integer>();
        while (rs.next()) {
          ret.add(rs.getInt(1));
        }
        return ret;
      } finally {
        rs.close();
      }
    } catch (final SQLException e) {
      throw new TransactionException(e);
    } catch (final ConnectionException e) {
      throw new TransactionException(e);
    }
  }
}
