package org.caosdb.server.query;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import java.io.IOException;
import org.caosdb.server.CaosDBServer;
import org.junit.BeforeClass;
import org.junit.Test;

public class QueryTest {

  @BeforeClass
  public static void initServerProperties() throws IOException {
    CaosDBServer.initServerProperties();
  }

  String getCacheKey(String query) {
    Query q = new Query(query);
    q.parse();
    return q.getCacheKey();
  }

  @Test
  public void testGetKey() {
    assertEquals("enamePOV(pname,=,val1)", getCacheKey("FIND ename WITH pname = val1"));
    assertEquals("enamePOV(pname,=,val1)", getCacheKey("COUNT ename WITH pname = val1"));
    assertEquals("enamePOV(pname,=,val1)", getCacheKey("SELECT bla FROM ename WITH pname = val1"));
    assertEquals("enamePOV(pname,null,null)", getCacheKey("SELECT bla FROM ename WITH pname"));
    assertEquals(
        "enamemaxPOV(pname,null,null)",
        getCacheKey("SELECT bla FROM ename WITH THE GREATEST pname"));

    assertEquals(
        "RECORDenamePOV(pname,=,val1)", getCacheKey("FIND RECORD ename WITH pname = val1"));
    assertEquals("ENTITYPOV(pname,=,val1)", getCacheKey("COUNT ENTITY WITH pname = val1"));
    assertEquals(
        "enameConj(POV(pname,=,val1)POV(ename2,=,val2))",
        getCacheKey("SELECT bla FROM ename WITH pname = val1 AND ename2 = val2"));

    assertEquals("versionedENTITYID(,>,2)", getCacheKey("FIND ANY VERSION OF ENTITY WITH ID > 2"));
    assertEquals("ENTITYID(min,,)", getCacheKey("FIND ENTITY WITH THE SMALLEST ID"));
    assertEquals("ENTITYSAT(asdf/%%)", getCacheKey("FIND ENTITY WHICH IS STORED AT /asdf/*"));
    assertEquals("ENTITYSAT(asdf/asdf)", getCacheKey("FIND ENTITY WHICH IS STORED AT asdf/asdf"));
    assertEquals(
        "enamePOV(ref1,null,null)SUB(POV(pname,>,val1)",
        getCacheKey("FIND ename WITH ref1 WITH pname > val1 "));
    assertEquals(
        "ename@(ref1,null)SUB(POV(pname,>,val1)",
        getCacheKey("FIND ename WHICH IS REFERENCED BY ref1 WITH pname > val1 "));
  }

  @Test
  public void testOptimizationOfFindStar() {
    Query q = new Query("FIND *");
    q.parse(false);
    assertEquals("*", q.getEntity().str);
    assertNull(q.getRole());

    q.optimize();
    assertNull(q.getEntity());
    assertEquals(Query.Role.ENTITY, q.getRole());
  }
}
